package Epoll::DB::Poll::Type::Binary;

# $Id: Binary.pm 424 2010-10-06 23:49:19Z nanardon $

use strict;
use warnings;
use base 'Epoll::DB::Poll';
use YAML;

=head1 NAME

Epoll::DB::Poll::Results - Epoll computation module

=head1 DESCRIPTION

Epoll module

=cut

sub _check_ballot {
    my ($self, $bdata) = @_;

}

sub _compute_results {
    my ($self) = @_;
    $self->_results(
        $self->info('empty_ballot_has_voice')
        ? 0
        : 1
    );
}

sub absolute_majority {
    my ($self) = @_;
    return int($self->_voices_ballot_count / 2) + 1;
}    

sub ___abstention {
    my ($self) = @_;
    
    my $sth = $self->db->prepare_cached(
        q{
        select count(*) from ballot where poll = ?
        and id not in (select id from ballot_item) and
        (invalid = 'false' or invalid is null)
        }
    );

    $sth->execute($self->voteid);
    my $res = $sth->fetchrow_hashref;
    $sth->finish;
    $res->{count}
}

sub voices_ballot_count { _voices_ballot_count(@_) }

sub _voices_ballot_count {
    my ($self) = @_;
    return $self->info('empty_ballot_has_voice')
        ? $self->valid_ballot_count
        : $self->not_empty_ballot_count;
}

sub _results {
    my ($self, $nonull) = @_;

    my $sth = $self->db->prepare(
        q{
        select count(ballot.id), value from
        (
        select NULL as id, lower(label) as value from choice where poll = $1
        union
        select ballot.id, coalesce(ballot_map.to, value) as "value" from ballot
        } . ($nonull ? '' : ' left ') . q{ join ballot_item
        on ballot.id = ballot_item.id 
        left join ballot_map on ballot_map.from = ballot_item.value and
        ballot_map.poll = ballot.poll
        where ballot.poll = $1 and (invalid = 'false' or invalid is null
        )
        group by ballot.id, coalesce(ballot_map.to, value)) as ballot
        group by value
        order by count desc, value
        }
    );
    $sth->execute($self->voteid);
    my @results;
    my $abs_maj = $self->absolute_majority;
    my $wanted_count = $self->info('elected_count');
    my $voice_count = $self->_voices_ballot_count;
    while (my $res = $sth->fetchrow_hashref) {
        if ($res->{count} >= $abs_maj) {
            $res->{abs_maj} = 1;
        }
        if (scalar(@results) < $wanted_count) {
            $res->{elected} = 1 if ($res->{count});
        } elsif (scalar(@results)) {
            $res->{elected} = $results[-1]->{elected}
                if($res->{count} == $results[-1]->{count});
        }
        if (scalar(@results)) {
            $res->{order} = ($res->{count} == $results[-1]->{count}
                ? $results[-1]->{order}
                : scalar(@results) + 1);
        } else { $res->{order} = 1 }
        $res->{voices_percent} = $res->{count} / $voice_count * 100
            if($voice_count); # avoid divide by 0

        push(@results, $res);
    }
    \@results;
}


=head1 AUTHOR

Thauvin Olivier

=head1 LICENSE

This library is free software, you can redistribute it and/or modify
it under the same terms as Perl itself or CeCILL.

=cut

1;
